#ifndef QMPLAYER_H
#define QMPLAYER_H

/*
 *
 * qmplayer - A Qt class for embedding MPlayer
 * Copyright (C) 2011 by Israel Lins Albuquerque
 *
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QObject>
#include <QProcess>
#include <QSize>
#include <QTimer>
#include <QHash>
#include <QPair>
#include <QFlags>

class QMPlayer : public QObject
{
    Q_OBJECT

public:
    enum State {
        stStopped,
        stLoading,
        stBuffering,
        stPlaying,
        stPaused
    };

    enum Mode {
        mdAuto = -1,
        mdEmbeddedMode = 0,
//        mdPipeMode
    };

    enum Parameter {
        paNone              = 0x0000,

        // Time in ms
        paAudioDelay        = 0x0001,
        // Decibels percent 0 to 100
        paAudioVolume       = 0x0002,
        paAudioMute         = 0x0004,

        // a_perc => -100 <-> 100 default 0
        paVideoBrightness    = 0x0008,
        paVideoContrast      = 0x0010,
        paVideoGamma         = 0x0020,
        paVideoHue           = 0x0040,
        paVideoSaturation    = 0x0080
    };
    Q_DECLARE_FLAGS (Parameters, Parameter);

    enum ErrType {
        etNoErr,
        etFatal,
        etWarning,
        etOpNotPermited,
        etPlaybackInterrupted,
        etUnknown
    };

    struct MediaInfo {
        QString url;

        struct VideoInfo {
            QString codec;
            QString format;
            qint32 bitrate;
            QSize size;
            qreal fps;

            VideoInfo() : format(), bitrate(0), size(), fps(0) {}
        } video;

        struct AudioInfo {
            QString codec;
            QString format;
            qint32 bitrate;
            qint32 sampleRate;
            qint32 numChannels;

            AudioInfo() : format(), bitrate(0), sampleRate(0), numChannels(0) {}
        } audio;

        QHash<QString, QString> tags;

        bool valid;
        qint64 length;
        qint64 startTime;
        bool seekable;

        MediaInfo(const QString& a_url = QString()) : url(a_url), video(), audio(), tags(), valid(false), length(0), startTime(0), seekable(false) {}

        bool hasVideo() const { return !video.codec.isEmpty(); }
        bool hasAudio() const { return !audio.codec.isEmpty(); }
    };

public:
    explicit QMPlayer(QObject* a_parent = 0);
    virtual ~QMPlayer();

    // process control
    void setWinID(qint32 a_winId);

    void setCustomArgs(const QStringList& a_args = QStringList());

private:
    void writeCommand(QString a_cmd);

public:

    // info
    QProcess::ProcessState processState() const;
    QMPlayer::State state() const;

    // audio
    qint32 audioDelay() const;
    qint32 audioVolume() const;
    bool audioMute() const;

    // video
    qint32 videoBrightness() const;
    qint32 videoContrast() const;
    qint32 videoGama() const;
    qint32 videoHue() const;
    qint32 videoSaturation() const;

    // media
    qint64 tell() const;

    const QMPlayer::MediaInfo& mediaInfo() const;

    static void setMPlayerPath(const QString& a_path);
    static QString mPlayerPath();
    static QString mPlayerVersion();

public slots:
    // audio
    void setAudioDelay(qint32 a_ms, bool a_absolute = true);
    void setAudioVolume(qint32 a_db, bool a_absolute = true);
    void setAudioMute(bool a_mute);

    // video
    void setVideoBrightness(qint32 a_perc, bool a_absolute = true);
    void setVideoContrast(qint32 a_perc, bool a_absolute = true);
    void setVideoGamma(qint32 a_perc, bool a_absolute = true);
    void setVideoHue(qint32 a_perc, bool a_absolute = true);
    void setVideoSaturation(qint32 a_perc, bool a_absolute = true);

    // media
    void play(const QString& a_url);
    void pause();
    void stop();
    void seek(qint64 a_value, bool a_absolute = true);

    QPair<QMPlayer::ErrType, QString> lastError();

private:
    void setError(QMPlayer::ErrType a_type, const QString& a_error);
    void setState(QMPlayer::State a_new);
    void setParameter(Parameter a_param, qint32 a_v, bool a_absolute);

private slots:
    void sendSeek();
    void sendPendingParameters();
    void emitErrors();
    void askPlaybackProgress();

    void processFinished(int, QProcess::ExitStatus);
    void processReadyReadStandardError();
    void processReadyReadStandardOutput();
    void parseMediaInfo(QString a_line);
    void parsePosition(QString a_line);

signals:
    void tick(qint64 a_currentTime);
    void stateChange(QMPlayer::State a_new, QMPlayer::State a_old);
    void mediaInfoChange();
    void error(QMPlayer::ErrType a_type, const QString& a_error);

    void parameterChange(QMPlayer::Parameter a_p, qint32 a_newValue);

private:
    QProcess m_process;
    QStringList m_customArgs;
    Mode m_mode;

    qint32 m_winId;
    MediaInfo m_mediaInfo;
    qint64 m_currentTime;
    QString m_seekCmd;
    QTimer m_seekTimer;

    State m_state;
    QTimer m_askPlaybackProgress;
    qint32 m_askPlaybackProgressTimes;

    QTimer m_notifyErrors;

    QHash<Parameter, qint32> m_parameterValues;
    Parameters m_pendingParameters;

    QPair<QMPlayer::ErrType, QString> m_error;

    static QString sm_mplayerPath;
    static QString sm_mplayerVersion;
};

Q_DECLARE_OPERATORS_FOR_FLAGS (QMPlayer::Parameters);

#endif // QMPLAYER_H
